tr.tr_passed, div.tr_passed {
    background-color: rgba(0,250,0,0.2) !important;
}

tr.tr_failure, div.tr_failure {
    background-color: rgba(250,0,0,0.2) !important;
}

tr.tr_skipped{
    background-color: rgba(0,0,0,0.1) !important;
}

td.status p {
    padding: 3px 5px;
    text-align: center;
    border-radius: 10px;
}

td.status p.tr_passed{
    border: 1px solid #008000;
    color: #008000;
}

td.status p.tr_failure{
    border: 1px solid #800000;
    color: #800000;
}

td.status p.tr_skipped{
    border: 1px solid #555;
    color: #555;
}
